@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2015 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2017 Carsten Gnoerlich.
  *
  *  Email: carsten@dvdisaster.org  -or-  cgnoerlich@fsfe.org
  *  Project homepage: http://www.dvdisaster.org
@@ -55,9 +55,9 @@ void FreeBitmap(Bitmap *bm)
  * Count the '1' bits in the bitmap 
  */
 
-int CountBits(Bitmap *bm)
-{ int i;
-  int sum = 0;
+gint32 CountBits(Bitmap *bm)
+{ gint32 i;
+  gint32 sum = 0;
 
   for(i=0; i<bm->size; i++)
     if(GetBit(bm, i))
