@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2015 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2017 Carsten Gnoerlich.
  *
  *  Email: carsten@dvdisaster.org  -or-  cgnoerlich@fsfe.org
  *  Project homepage: http://www.dvdisaster.org
@@ -537,7 +537,10 @@ process_ecc:
    if(image && !image->eccFile)
    {  
       switch(image->eccFileState)
-      {  case ECCFILE_MISSING:
+      {  case ECCFILE_NOPERM:
+	    PrintLog(_("permission denied\n"));
+	    break;
+	 case ECCFILE_MISSING:
 	    PrintLog(_("not present\n"));
 	    break;
 	 case ECCFILE_INVALID:
@@ -569,47 +572,34 @@ process_ecc:
    if(eh->creatorVersion)
    {  int major = eh->creatorVersion/10000; 
       int minor = (eh->creatorVersion%10000)/100;
-      int pl    = eh->creatorVersion%100;
+      int micro = eh->creatorVersion%100;
+      char *unstable="";
 
-      if(eh->creatorVersion%100)        
-      {  char *format, *color_format = NULL;
-
-	 if(eh->creatorVersion < 6000) format = "%s-%d.%d.%d";
-	 else if(eh->creatorVersion <= 6500) format = "%s-%d.%d (pl%d)";
-	 else
-	 {  if(eh->methodFlags[3] & MFLAG_DEVEL) 
-	    {  format = "%s-%d.%d (devel-%d)";
-	       color_format = "%s-%d.%d <span %s>(devel-%d)</span>";
-	    }
-	    else if(eh->methodFlags[3] & MFLAG_RC) 
-	    {  format = "%s-%d.%d (rc-%d)";
-	       color_format = "%s-%d.%d <span %s>(rc-%d)</span>";
-	    }
-	    else format = "%s-%d.%d (pl%d)";
-	 }
-	 PrintLog(format, _("created by dvdisaster"), major, minor, pl);
+      /* Suppress (unstable) output in debug mode to facilitate regression tests */
+      if((eh->methodFlags[3] & MFLAG_DEVEL) && !Closure->regtestMode)
+	unstable=" (unstable)";
+      
+      if(micro)  /* version format x.xx.x */
+      {  char *format = "%s-%d.%d.%d%s";
+	 PrintLog(format, _("created by dvdisaster"), major, minor, micro, unstable);
 	 PrintLog("\n");
 
 	 if(Closure->guiMode)
-	 {  if(color_format)
-	      SwitchAndSetFootline(wl->cmpEccNotebook, 1,
-				   wl->cmpEccCreatedBy, 
-				   color_format, "dvdisaster",
-				   major, minor, Closure->redMarkup, pl);
-	    else
-	      SwitchAndSetFootline(wl->cmpEccNotebook, 1,
-				   wl->cmpEccCreatedBy, 
-				   format, "dvdisaster",
-				   major, minor, pl);
+	 {  SwitchAndSetFootline(wl->cmpEccNotebook, 1,
+				 wl->cmpEccCreatedBy, 
+				 format, "dvdisaster",
+				 major, minor, micro, unstable);
 	 }
       }
-      else
-      {  PrintLog(_("created by dvdisaster-%d.%d\n"), 
-		  major, minor);
+      else  /* version format x.xx */
+      {  char *format = "%s-%d.%d%s";
+	 PrintLog(format, _("created by dvdisaster"), 
+		 major, minor, unstable);
+	 PrintLog("\n");
 	 if(Closure->guiMode)
 	   SwitchAndSetFootline(wl->cmpEccNotebook, 1,
-				wl->cmpEccCreatedBy, "dvdisaster-%d.%d",
-				major, minor);
+				wl->cmpEccCreatedBy, format, "dvdisaster",
+				major, minor, unstable);
       }
    }
    else
@@ -830,11 +820,11 @@ process_ecc:
       {  if(!Closure->guiMode)
 	      PrintProgress(_("- ecc md5sum       : %3d%%"),percent);
 	 else SetLabelText(GTK_LABEL(wl->cmpEccMd5Sum), "%3d%%", percent);
-	      last_percent = percent;
+	 last_percent = percent;
       }
 
       if(Closure->stopActions)   
-      {  if(Closure->stopActions == STOP_CURRENT_ACTION) /* suppress memlead warning when closing window */
+      {  if(Closure->stopActions == STOP_CURRENT_ACTION) /* suppress memleak warning when closing window */
 	    SetLabelText(GTK_LABEL(wl->cmpEccResult), 
 			 _("<span %s>Aborted by user request!</span>"), Closure->redMarkup); 
 	 goto terminate;
